from .Store import *
from .DirManager import *
from .WebRequests import *
from .ExcelParser import *
from .JsonAdapter import *

__version__ = '1.0'
__author__ = 'bonkibon75@gmail.com'
__desc__ = 'Automatic schedule for Kursk State Polytechnic College (KSPC)'
